# This code computes several statistical variables e.g. RMSE, BIAS, OBAR, FBAR etc against any analysis using the series_analysis tool of the Model Evaluation Tool (MET). The input files should be in GRIB files. This is prepared for use with the HWRF model output but can be used for any other model comparisons.  

#Usage: python run_series_analysis.py [--storm 09l [--cycle 2017082400]] [--v 1] >&! run_series_analysis.out&
#if experience run-time errors involving the loading of shared libraries, do 'module purge', then reissue 'module use /contrib/modulefiles; module load met/7.0'
#example Experimental and Control filename: maria15l.2017093012.hwrfprs.global.0p25.f120.grb2; 
#example analysis (e.g. GFS) filename: gfs.t18z.pgrb2.20171013.0p25.anl

# Note is that for your use, customization is required by editing paths, for example.
# This script was prepared by John Henderson, AER jhenders@aer.com

# The use of the script is "as is" basis. Neither Developmental testbed Center (DTC) nor the author accept any liability and distributed without any warranty 

from datetime import datetime
from datetime import timedelta
import os
import sys

try: import argparse
except: from lib import argparse

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("-storm number, e.g., al09","--storm", help="storm number, e.g., al09",default='all')             #optional
    parser.add_argument("-storm cycle, e.g., 2017093000","--cycle", help="storm cycle, .e.g., 2017093000",default='all')  #optional
    parser.add_argument("-v","--v", help="set to one for ob selection printout",type=int)
    args = parser.parse_args()


h18cdir='/YOUR/PATH/TO/CONTROL/DATA'
h18gdir='/YOUR/PATH/TO/EXPERIMENTAL/DATA'
gfsdir='/YOUR/PATH/TO/ANALYSIS/DATA'
## Fields and levels to calculate Statistics
fields=['TMP','HGT','RH','UGRD','VGRD','WIND']
levels=['P1000','P925','P850','P700','P500','P300','P200']
## Forecast lead times
forecastlengths=['f024','f048']

searchcmd=''
if args.storm!='all':
   searchcmd=args.storm #one individual storm, such as '09l','11l','15l''] #09=harvey,11=irma,12=jose,15=maria,17=ophelia
if args.cycle!='all':
   searchcmd=searchcmd+'.'+args.cycle
if searchcmd=='':
   searchcmd='hwrf'
print "Running for all files that contain ",searchcmd+'.'

verbosityflag=1
configfiletemplatepath='/YOUR/PATH/TO/SeriesAnalysis/CONFIG/FILE/TEMPLATE'
configfiletemplate='SeriesAnalysisConfig2017-template' #note: script keys in on dash in this filename

##################################################################################################################
configfiletemplateroot,notused=configfiletemplate.split('-')

#Loop over each Experimental forecast and accumulate corresponding Control and Analysis files:
for forecastlength in forecastlengths:
   h18gfilelistname='h18gfilelist-'+forecastlength
   h18cfilelistname='h18cfilelist-'+forecastlength
   gfsfilelistname='gfsfilelist-'+forecastlength
   h18gfilelist=[]
   h18cfilelist=[]
   gfsfilelist=[]
   for h18gfilebase in os.listdir(h18gdir):
      #match desired storms:
      if searchcmd in h18gfilebase:
         if forecastlength in h18gfilebase:
            #Pick apart filename to get valid time:
            stormidname,cycle,filetype,region,res,fxl,extension=h18gfilebase.split(".")
            dateinitial= datetime(int(cycle[0:4]),int(cycle[4:6]),int(cycle[6:8]),int(cycle[8:10])) 
            datevalid = datetime(int(cycle[0:4]),int(cycle[4:6]),int(cycle[6:8]),int(cycle[8:10])) + timedelta(hours=int(fxl[1:]))
            #example 18R/18C filename: maria15l.2017093012.hwrfprs.global.0p25.f120.grb2; example GFS analysis filename: gfs.t18z.pgrb2.20171013.0p25.anl

            h18gfile=os.path.join(h18gdir,h18gfilebase)
            h18cfile=os.path.join(h18cdir,h18gfilebase)
            datevalidstring="%04d"%datevalid.year+"%02d"%datevalid.month+"%02d"%datevalid.day
#            gfsfile=os.path.join(gfsdir,'gfs.t'+"%02d"%datevalid.hour+'z.pgrb2.'+datevalidstring+'.0p25.anl')
            gfsfile=os.path.join(gfsdir,'gfs.'+datevalidstring+"%02d"%datevalid.hour+'.pgrb2.anl')
            if args.v:
               print "Full 18R,18C,GFS files:",h18gfile,h18cfile,gfsfile
 
            if os.path.exists(h18gfile) and os.path.exists(h18cfile) and os.path.exists(gfsfile):
               if args.v:
                  os.system("ls -l "+h18gfile+' '+h18cfile+' '+gfsfile)

               h18gfilelist.append(os.path.join(h18gdir,h18gfile))
               h18cfilelist.append(os.path.join(h18cdir,h18cfile))
               gfsfilelist.append(os.path.join(gfsdir,gfsfile))
            else:
               print "---------"
               if args.v:
                  os.system("ls -l "+h18gfile+' '+h18cfile+' '+gfsfile)
               print "One or more files is not available above"
               print "---------"


      #WRITE file lists:
      with open(h18gfilelistname,'w') as file:
         for item in h18gfilelist:
            file.write("%s" % item+"\n")
      with open(h18cfilelistname,'w') as file:
         for item in h18cfilelist:
            file.write("%s" % item+"\n")
      with open(gfsfilelistname,'w') as file:
         for item in gfsfilelist:
            file.write("%s" % item+"\n")

   #sys.exit()
   #loop over all fields and levels, then call series_analysis using above file lists
   for field in fields:
      for level in levels:
         #update series_analysis template
         with open(os.path.join(configfiletemplatepath,configfiletemplate),'r') as file:
            filedata=file.read()

         filedata=filedata.replace('FFFF',field)
         filedata=filedata.replace('LLLL',level)
         configfile=configfiletemplateroot+'-'+forecastlength+'-'+field+'-'+level
         with open(configfile,'w') as file:
            file.write(filedata)

         #call series_analysis
         print 'Calling series_analysis for '+field+' at '+level
## Change output file names below
         if searchcmd!='':
            h18gcmd='series_analysis -fcst '+h18gfilelistname+' -obs '+gfsfilelistname+' -v '+str(verbosityflag)+' -config '+configfile+' -paired -out h18g.'+forecastlength+'.'+field+'.'+level+'.'+searchcmd+'.nc '
            h18ccmd='series_analysis -fcst '+h18cfilelistname+' -obs '+gfsfilelistname+' -v '+str(verbosityflag)+' -config '+configfile+' -paired -out h18c.'+forecastlength+'.'+field+'.'+level+'.'+searchcmd+'.nc '
         else:
            h18gcmd='series_analysis -fcst '+h18gfilelistname+' -obs '+gfsfilelistname+' -v '+str(verbosityflag)+' -config '+configfile+' -paired -out h18g.'+forecastlength+'.'+field+'.'+level+'.'+searchcmd+'nc '
            h18ccmd='series_analysis -fcst '+h18cfilelistname+' -obs '+gfsfilelistname+' -v '+str(verbosityflag)+' -config '+configfile+' -paired -out h18c.'+forecastlength+'.'+field+'.'+level+'.'+searchcmd+'nc '
         if args.v:
            print "h18gcmd",h18gcmd
            print "h18ccmd",h18ccmd

         os.system(h18gcmd)
         os.system(h18ccmd)
             
